import math
import numpy as np
import torch

def euclidean_dist(pointA, pointB):
    if(len(pointA) != len(pointB)):
        raise Exception("expected point dimensionality to match")
    total = float(0)
    for dimension in range(0, len(pointA)):
        total += (pointA[dimension] - pointB[dimension])**2
    return math.sqrt(total)

def gaussian_kernel(distance, bandwidth):
    euclidean_distance = np.sqrt(((distance)**2).sum(axis=1))
    val = (1/(bandwidth*math.sqrt(2*math.pi))) * np.exp(-0.5*((euclidean_distance / bandwidth))**2)
    return val

def dense_kernel(distance, dense, bandwidth):
    euclidean_distance = torch.norm(distance, dim=1)
    val = dense.clone()
    ids = euclidean_distance > bandwidth
    val[ids] = 0.0
    return val


def multivariate_gaussian_kernel(distances, bandwidths):

    
    dim = len(bandwidths)

    
    cov = np.multiply(np.power(bandwidths, 2), np.eye(dim))

    
    exponent = -0.5 * np.sum(np.multiply(np.dot(distances, np.linalg.inv(cov)), distances), axis=1)
    val = (1 / np.power((2 * math.pi), (dim/2)) * np.power(np.linalg.det(cov), 0.5)) * np.exp(exponent)

    return val
